<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Checkout_Field {

  public $ID = 0;
  public $post_author = 0;
  public $post_date = '0000-00-00 00:00:00';
  public $post_date_gmt = '0000-00-00 00:00:00';
  public $post_content = '';
  public $post_title = '';
  public $post_excerpt = '';
  public $post_status = 'publish';
  public $comment_status = 'open';
  public $ping_status = 'open';
  public $post_password = '';
  public $post_name = '';
  public $to_ping = '';
  public $pinged = '';
  public $post_modified = '0000-00-00 00:00:00';
  public $post_modified_gmt = '0000-00-00 00:00:00';
  public $post_content_filtered = '';
  public $post_parent = 0;
  public $guid = '';
  public $menu_order = 0;
  public $post_mime_type = '';
  public $comment_count = 0;
  public $filter;

  private $unique_key;
  private $label;
  private $placeholder;
  private $is_required;

  private $wp_post;
  private $meta_prefix;

  public function __construct($id) {
    $this->meta_prefix = '_quickcab_checkout_field_';
    $checkout_field = WP_Post::get_instance($id);

    if ( $checkout_field ) {
      $this->setup_checkout_field($checkout_field);
    } else {
      return false;
    }
  }

  /**
   * Return the ID of the WP_Post object
   *
   * @since  1.0
   * @return int  order ID
   */
  public function get_ID() {
    return $this->ID;
  }

  /**
   * Return the unique key
   *
   * @since  1.0
   * @return string  the unique key
   */
  public function get_unique_key() {
    return apply_filters('quickcab_checkout_field_get_unique_key', $this->unique_key, $this->get_ID());
  }

  /**
   * Return the label
   *
   * @since  1.0
   * @return string  the label
   */
  public function get_label() {
    return apply_filters('quickcab_checkout_field_get_label', $this->label, $this->get_ID());
  }

  /**
   * Return the placeholder
   *
   * @since  1.0
   * @return string  the placeholder
   */
  public function get_placeholder() {
    return apply_filters('quickcab_checkout_field_get_placeholder', $this->placeholder, $this->get_ID());
  }

  /**
   * Return the input type
   *
   * @since  1.0
   * @return string  the input type
   */
  public function get_input_type() {
    return apply_filters('quickcab_checkout_field_get_input_type', $this->input_type, $this->get_ID());
  }

  /**
   * Return the input options
   *
   * @since  1.0
   * @return string  the input options
   */
  public function get_input_options() {
    return apply_filters('quickcab_checkout_field_get_input_options', $this->input_options, $this->get_ID());
  }

  /**
   * Return the input options, formatted for woocommerce
   *
   * @since  1.0
   * @return string  the input options
   */
  public function get_input_options_woocommerce() {
    $options = $this->get_input_options();

    $woocommerce_options = array();

    if ( !$this->is_required() ) {
      $woocommerce_options['none'] = __('Select an option', 'quickcab') . '...';
    }

    foreach ( $options as $option ) {
      $woocommerce_options[$option] = $option;
    }

    return $woocommerce_options;
  }

  /**
   * Is required?
   *
   * @since  1.0
   * @return bool  is required?
   */
  public function is_required() {
    return apply_filters('quickcab_checkout_field_get_is_required', $this->is_required, $this->get_ID());
  }

  /**
   * Show in email?
   *
   * @since  1.0
   * @return bool  show in email?
   */
  public function show_in_email() {
    return apply_filters('quickcab_checkout_field_get_show_in_email', $this->show_in_email, $this->get_ID());
  }

  /**
   * Gets all QuickCab_Surge_Price_Rule
   *
   * @since  1.0
   * @return array  array of all checkout fields
   */
  public static function find_all() {
    $checkout_fields = array();

    $post_search = new WP_Query(array(
      'post_type' => 'qc_checkout_field',
      'post_staus' => array('publish'),
      'posts_per_page' => -1
    ));

    $unacceptable_status = apply_filters('quickcab_checkout_field_unacceptable_status', array(
      'auto-draft', 'future', 'draft', 'private', 'trash'
    ));

    foreach ( $post_search->posts as $checkout_field ) {
      if ( in_array($checkout_field->post_status, $unacceptable_status) ) {
        continue;
      }

      $checkout_fields[$checkout_field->ID] = new QuickCab_Checkout_Field($checkout_field->ID);
    }

    return $checkout_fields;
  }

  /**
   * Get the meta data for the checkout field
   *
   * @since  1.0
   * @param  string  $key     the key to fetch
   * @param  boolean $single  get single data or array
   * @return [array|string]   value of meta data if single = true, array if not
   */
  public function get_meta($key = '', $single = true) {
    $value = get_post_meta($this->get_ID(), $this->meta_prefix . $key, $single);
    return apply_filters('quickcab_checkout_field_get_meta_' . $key, $value, $this->get_ID(), $single);
  }

  /**
   * Update the meta data for the checkout field
   *
   * @since  1.0
   * @param  string $key         the key to update
   * @param  string $value       the new value
   * @param  string $prev_value  the old value
   * @return [int|bool]          meta ID if key didn't exist, true on success, false on fail
   */
  public function update_meta($key = '', $value = '', $prev_value = '') {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    $value = apply_filters('quickcab_checkout_field_update_meta_' . $key, $value, $this->get_ID());
    return update_post_meta($this->get_ID(), $key, $value, $prev_value);
  }

  /**
   * Delete the meta data for the fixed address
   *
   * @since  1.0
   * @param  string $key  the key to delete
   * @return bool         success or not
   */
  public function delete_meta($key) {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    return delete_post_meta($this->get_ID(), $key);
  }

  /**
   * Populates the default WordPress Post fields to QuickCab_Checkout_Field
   *
   * @since  1.0
   * @param  WP_Post $post  the WP_Post
   * @return bool           if the save was successful
   */
  private function populate_post_fields($post) {
    if ( !is_object($post) || !is_a($post, 'WP_Post') || $post->post_type !== 'qc_checkout_field' ) {
      return false;
    }

    foreach ( $post as $key => $value ) {
      $this->$key = $value;
    }

    return true;
  }

  /**
   * Setup the object variables
   *
   * @since  1.0
   * @param  WP_Post $checkout_field  the checkout field WP_Post
   * @return bool                     did the setup suceed?
   */
  private function setup_checkout_field($checkout_field) {
    if ( is_null($checkout_field) || !is_a($checkout_field, 'WP_Post') || $checkout_field->post_type !== 'qc_checkout_field' ) {
      return false;
    }

    do_action('quickcab_before_setup_checkout_field', $this, $checkout_field);

    $this->ID = $checkout_field->ID;
    $this->wp_post = $checkout_field;

    $this->unique_key    = $this->setup_unique_key();
    $this->label         = $this->setup_label();
    $this->placeholder   = $this->setup_placeholder();
    $this->is_required   = $this->setup_is_required();
    $this->input_type    = $this->setup_input_type();
    $this->input_options = $this->setup_input_options();
    $this->show_in_email = $this->setup_show_in_email();

    $this->populate_post_fields($this->wp_post);

    do_action('quickcab_after_setup_checkout_field', $this, $checkout_field);

    return true;
  }

  private function setup_unique_key() {
    return $this->get_meta('unique_key');
  }

  private function setup_label() {
    return $this->get_meta('label');
  }

  private function setup_placeholder() {
    return $this->get_meta('placeholder');
  }

  private function setup_is_required() {
    return (bool) $this->get_meta('is_required');
  }

  private function setup_show_in_email() {
    return (bool) $this->get_meta('show_in_email');
  }

  private function setup_input_type() {
    return $this->get_meta('input_type');
  }

  private function setup_input_options() {
    return unserialize($this->get_meta('input_options'));
  }

}
