<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Journey {

  const METERS_TO_MILES = 0.00062137;
  const METERS_TO_KM = 0.001;

  private $origin;
  private $destination;
  private $waypoints;

  private $departure_time;
  private $direction;
  private $no_of_occupants = 0;
  private $no_of_suitcases = 0;
  private $booking_form;

  private $duration_in_seconds = 0;
  private $duration_in_seconds_traffic = 0;
  private $distance_in_meters = 0;
  private $base_to_collection_time;
  private $destination_to_base_time;
  private $extra_waiting_time;

  private $has_error = false;
  private $error_message = "";

  private $return_journey;

  public function __construct($options) {
    $this->initialize_journey($options);
  }

  public function get_origin_object() {
    return $this->origin;
  }

  public function get_destination_object() {
    return $this->destination;
  }

  public function get_origin_coords() {
    return array(
      'lat' => $this->origin['lat'],
      'lng' => $this->origin['lng']
    );
  }

  public function get_destination_coords() {
    return array(
      'lat' => $this->destination['lat'],
      'lng' => $this->destination['lng']
    );
  }

  public function get_waypoints() {
    return $this->waypoints;
  }

  /**
   * Returns the departure time of the journey
   *
   * @since   1.0
   * @return  int     the departure time (Unix Epoch)
   */
  public function get_departure_time() {
    return $this->departure_time;
  }

  /**
   * Returns the estimated arrival time for the journey
   *
   * @since   1.0
   * @return  int     the estimated arrival time (Unix Epoch)
   */
  public function get_estimated_arrival_time() {
    return $this->get_departure_time() + $this->get_duration_in_seconds();
  }

  /**
   * Returns the direction of the journey
   *
   * @since   1.0
   * @return  string   the direction  [one_way|return]
   */
  public function get_direction() {
    return $this->direction;
  }

  /**
   * Is the journey one way?
   *
   * @since   1.0
   * @return  bool     is the journey one way?
   */
  public function is_one_way() {
    return $this->get_direction() == 'one_way';
  }

  /**
   * Is the journey return?
   *
   * @since   1.0
   * @return  bool     is the journey return?
   */
  public function is_return() {
    return $this->get_direction() == 'return';
  }

  /**
   * Return the number of occupants for the journey
   *
   * @since   1.0
   * @return  int      the number of occupants
   */
  public function get_no_of_occupants() {
    return $this->no_of_occupants;
  }

  public function get_no_of_suitcases() {
    return $this->no_of_suitcases;
  }

  public function get_duration_in_seconds() {
    return empty($this->duration_in_seconds_traffic) ? $this->duration_in_seconds : $this->duration_in_seconds_traffic;
  }

  public function get_duration_in_minutes() {
    return $this->get_duration_in_seconds() / 60;
  }

  public function get_duration_in_hours() {
    return $this->get_duration_in_minutes() / 60;
  }

  public function get_duration_in_words() {
    return quickcab_mins_to_text($this->get_duration_in_seconds() / 60);
  }

  public function get_distance_in_meters() {
    return $this->distance_in_meters;
  }

  public function get_distance_in_words() {
    return quickcab_meters_to_text($this->get_distance_in_meters());
  }

  public function get_distance_in_given_unit() {
    $units = quickcab_get_option('distance_units');

    if ( $units == 'miles' ) {
      return $this->get_distance_in_meters() * self::METERS_TO_MILES;
    }

    return $this->get_distance_in_meters() * self::METERS_TO_KM;
  }

  public function get_return_journey() {
    return $this->return_journey;
  }

  public function has_error() {
    return $this->has_error;
  }

  public function get_error_message() {
    return $this->error_message;
  }

  public function get_base_to_collection_time() {
    return $this->base_to_collection_time;
  }

  public function get_destination_to_base_time() {
    return $this->destination_to_base_time;
  }

  public function get_extra_waiting_time() {
    return $this->extra_waiting_time;
  }

  public function get_booking_form() {
    return $this->booking_form;
  }

  /**
   * Get the activation time of the vehicle, including time from base if
   * option is selected
   */
  public function get_vehicle_activation_timestamp() {
    $activation_timestamp = $this->get_departure_time();

    if ( quickcab_get_option('use_base_location_for_availability') == 'on' ) {
      $activation_timestamp -= $this->get_base_to_collection_time();
    }

    return $activation_timestamp;
  }

  /**
   * Get the deactivation time of the vehicle, including time to base if
   * option is selected
   */
  public function get_vehicle_deactivation_timestamp() {
    $deactivation_timestamp = $this->get_estimated_arrival_time();

    if ( quickcab_get_option('use_base_location_for_availability') == 'on' ) {
      $deactivation_timestamp += $this->get_destination_to_base_time();
    }

    return $deactivation_timestamp;
  }

  private function initialize_journey($options) {
    $units = quickcab_get_option('distance_units');

    $departure_date_str = $options['departure_date'] . ' ' . $options['departure_time'];

    $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
    $departure_time = DateTime::createFromFormat('d/m/Y H:i', $departure_date_str, $timezone);
    $current_time = new DateTime('now', $timezone);

    if ( !$departure_time ) {
      $this->has_error = true;
      $this->error_message = esc_html__('Incorrect time format.', 'quickcab');
      return false;
    }

    if ( $departure_time->format('U') < $current_time->format('U') ) {
      $this->has_error = true;
      $this->error_message = esc_html__('Time of collection cannot be in the past.', 'quickcab');
      return false;
    }

    $this->origin = $options['origin'];
    $this->destination = $options['destination'];
    $this->waypoints = isset($options['waypoints']) ? $options['waypoints'] : [];

    $this->departure_time = $departure_time->format('U');
    $this->direction = isset($options['direction']) && $options['direction'] == 'return' ? 'return' : 'one_way';
    $this->extra_waiting_time = intval($options['extra_waiting_time']);
    $this->booking_form = $options['booking_form'];
    
    if ( $this->booking_form->enable_passengers_field() ) {
      $this->no_of_occupants = intval($options['occupants']);
    }

    if ( $this->booking_form->enable_suitcases_field() ) {
      $this->no_of_suitcases = intval($options['suitcases']);
    }

    $origins = [];
    $destinations = [];

    // add origin
    $origin_lat_lng = $options['origin']['lat'] . ',' . $options['origin']['lng'];
    array_push($origins, $origin_lat_lng);

    // add waypoints
    foreach ( $this->waypoints as $waypoint ) {
      $waypoint_lat_lng = $waypoint['lat'] . ',' . $waypoint['lng'];
      array_push($origins, $waypoint_lat_lng);
      array_push($destinations, $waypoint_lat_lng);
    }

    // add destination
    $destination_lat_lng = $options['destination']['lat'] . ',' . $options['destination']['lng'];
    array_push($destinations, $destination_lat_lng);

    $gmaps_options = array(
      'origins' => join('|', $origins),
      'destinations' => join('|', $destinations),
      'departure_time' => $this->get_departure_time()
    );

    $journey_details = quickcab_get_journey_details($gmaps_options);

    if ( !empty($journey_details->error_message) ) {
      $this->has_error = true;
      $this->error_message = esc_html__('Google Maps API Error', 'quickcab') . ': ' . $journey_details->error_message;
      return false;
    }

    $total_distance = 0;
    $total_duration = 0;
    $total_duration_in_traffic = 0;

    for ( $i = 0; $i < count($journey_details->rows); $i++ ) {
      $leg = $journey_details->rows[$i]->elements[$i];

      // error in one of the legs
      if ( $leg->status !== 'OK' ) {
        $this->has_error = true;
        $this->error_message = '[' . $leg->status . '] ' . esc_html__('This route is invalid', 'quickcab');
        return false;
      }

      $total_distance += $leg->distance->value;
      $total_duration += $leg->duration->value;
      $total_duration_in_traffic += $leg->duration_in_traffic->value;
    }

    $this->distance_in_meters = $total_distance;
    $this->duration_in_seconds = $total_duration;
    $this->duration_in_seconds_traffic = $total_duration_in_traffic;

    if ( quickcab_get_option('use_base_location_for_availability') == 'on' ) {
      $base_location = quickcab_get_option('vehicle_base_location');

      // calculate time from base location to collection address
      $base_to_collection_options = array(
        'origins' => $base_location,
        'destinations' => $this->get_origin_coords()['lat'] . "," . $this->get_origin_coords()['lng'],
        'arrival_time' => $this->get_departure_time()
      );
      $base_to_collection_journey = quickcab_get_journey_details($base_to_collection_options);
      $base_to_collection_matrix_data = $base_to_collection_journey->rows[0]->elements[0];

      if ( $base_to_collection_matrix_data && $base_to_collection_matrix_data->status !== 'ZERO_RESULTS' ) {
        $this->base_to_collection_time = $base_to_collection_matrix_data->duration->value;
      }

      // calculate time from destination address to base location
      $destination_to_base_options = array(
        'origins' => $this->get_destination_coords()['lat'] . "," . $this->get_destination_coords()['lng'],
        'destinations' => $base_location,
        'departure_time' => $this->get_estimated_arrival_time()
      );
      $destination_to_base_journey = quickcab_get_journey_details($destination_to_base_options);
      $destination_to_base_matrix_data = $destination_to_base_journey->rows[0]->elements[0];

      if ( $destination_to_base_matrix_data && $destination_to_base_matrix_data->status !== 'ZERO_RESULTS' ) {
        $this->destination_to_base_time = $destination_to_base_matrix_data->duration->value;
      }
    }

    if ( $this->is_return() ) {
      $return_options = $options;
      $return_options['origin'] = $options['destination'];
      $return_options['destination'] = $options['origin'];
      $return_options['direction'] = 'one_way';
      $return_options['departure_date'] = $options['return_departure_date'];
      $return_options['departure_time'] = $options['return_departure_time'];
      unset($return_options['return_departure_date']);
      unset($return_options['return_departure_time']);

      $this->return_journey = new QuickCab_Journey($return_options);
    }
  }
}
