<?php

if ( !defined('ABSPATH') ) exit;

class QuickCab_Vehicle {

  // wordpress attributes
  public $ID = 0;
  public $post_author = 0;
  public $post_date = '0000-00-00 00:00:00';
  public $post_date_gmt = '0000-00-00 00:00:00';
  public $post_content = '';
  public $post_title = '';
  public $post_excerpt = '';
  public $post_status = 'publish';
  public $comment_status = 'open';
  public $ping_status = 'open';
  public $post_password = '';
  public $post_name = '';
  public $to_ping = '';
  public $pinged = '';
  public $post_modified = '0000-00-00 00:00:00';
  public $post_modified_gmt = '0000-00-00 00:00:00';
  public $post_content_filtered = '';
  public $post_parent = 0;
  public $guid = '';
  public $menu_order = 0;
  public $post_mime_type = '';
  public $comment_count = 0;
  public $filter;

  private $name;
  private $pricing_structure;
  private $starting_price;
  private $price_per_distance;
  private $price_per_distance_from_base;
  private $price_per_hour;
  private $price_per_minute;
  private $price_per_occupant;
  private $price_per_suitcase;
  private $price_per_waiting_unit;
  private $waiting_unit;
  private $occupant_space;
  private $suitcase_space;
  private $number_owned;
  private $calendar_color;

  private $wp_post;
  private $meta_prefix;

  public function __construct($id) {
    $this->meta_prefix = '_quickcab_vehicle_';
    $vehicle = WP_Post::get_instance($id);

    if ( $vehicle ) {
      $this->setup_vehicle($vehicle);
    } else {
      return false;
    }
  }

  /**
   * Return the ID of the WP_Post object
   *
   * @since  1.0
   * @return int  vehicle ID
   */
  public function get_ID() {
    return $this->ID;
  }

  /**
   * Return the name of the vehicle
   *
   * @since  1.0
   * @return string  name of vehicle
   */
  public function get_name() {
    return apply_filters('quickcab_vehicle_get_name', $this->name, $this->get_ID());
  }

  /**
   * Return the pricing structure of the vehicle
   *
   * @since  1.0
   * @return string  vehicle pricing structure
   */
  public function get_pricing_structure() {
    return apply_filters('quickcab_vehicle_get_pricing_structure', $this->pricing_structure, $this->get_ID());
  }

  /**
   * Return the starting price of the vehicle
   *
   * @since  1.0
   * @return float  vehicle starting price
   */
  public function get_starting_price() {
    return apply_filters('quickcab_vehicle_get_starting_price', $this->starting_price, $this->get_ID());
  }

  /**
   * Return the price per distance of the vehicle (the distance unit is defined
   * by the user in the settings)
   *
   * @since  1.0
   * @return float  vehicle starting price
   */
  public function get_price_per_distance() {
    return apply_filters('quickcab_vehicle_get_price_per_distance', $this->price_per_distance, $this->get_ID());
  }

  /**
   * Return the price per distance from base of the vehicle (the distance unit
   * is defined by the user in the settings)
   *
   * @since  1.0
   * @return float  vehicle starting price
   */
  public function get_price_per_distance_from_base() {
    return apply_filters('quickcab_vehicle_get_price_per_distance_from_base', $this->price_per_distance_from_base, $this->get_ID());
  }

  /**
   * Return the price per hour
   *
   * @since  1.0
   * @return float  price per hour
   */
  public function get_price_per_hour() {
    return apply_filters('quickcab_vehicle_get_price_per_hour', $this->price_per_hour, $this->get_ID());
  }

  /**
   * Return the price per minute
   *
   * @since  1.0
   * @return float  price per minute
   */
  public function get_price_per_minute() {
    return apply_filters('quickcab_vehicle_get_price_per_minute', $this->price_per_minute, $this->get_ID());
  }

  /**
   * Return the price per occupant
   *
   * @since  1.0
   * @return float  price per occupant
   */
  public function get_price_per_occupant() {
    return apply_filters('quickcab_vehicle_get_price_per_occupant', $this->price_per_occupant, $this->get_ID());
  }

  /**
   * Return the price per suitcase
   *
   * @since  1.0
   * @return float  price per suitcase
   */
  public function get_price_per_suitcase() {
    return apply_filters('quickcab_vehicle_get_price_per_suitcase', $this->price_per_suitcase, $this->get_ID());
  }

  /**
   * Return the price per waiting unit
   *
   * @since  1.0
   * @return float  price per waiting unit
   */
  public function get_price_per_waiting_unit() {
    return apply_filters('quickcab_vehicle_get_price_per_waiting_unit', $this->price_per_waiting_unit, $this->get_ID());
  }

  /**
   * Return the waiting unit
   *
   * @since  1.0
   * @return string  waiting unit
   */
  public function get_waiting_unit() {
    return apply_filters('quickcab_vehicle_get_waiting_unit', $this->waiting_unit, $this->get_ID());
  }

  /**
   * Return the waiting unit in translatable format
   *
   * @since  1.0
   * @return string  waiting unit
   */
  public function get_waiting_unit_i18n() {
    if ( $this->get_waiting_unit() == 'minutes' ) {
      $waiting_unit = esc_html__('minute', 'quickcab');
    } else {
      $waiting_unit = esc_html__('hour', 'quickcab');
    }

    return apply_filters('quickcab_vehicle_get_waiting_unit_i18n', $waiting_unit, $this->get_ID());
  }

  /**
   * Return the price per waiting min
   *
   * @since  1.0
   * @return float  price per waiting min
   */
  public function get_price_per_waiting_min() {
    if ( $this->get_waiting_unit() == 'hours' ) {
      return $this->get_price_per_waiting_unit() / 60;
    }

    return $this->get_price_per_waiting_unit();
  }

  /**
   * Return the occupant space
   *
   * @since  1.0
   * @return int  occupant space
   */
  public function get_occupant_space() {
    return apply_filters('quickcab_vehicle_get_occupant_space', $this->occupant_space, $this->get_ID());
  }

  /**
   * Return the suitcase space
   *
   * @since  1.0
   * @return int  suitcase space
   */
  public function get_suitcase_space() {
    return apply_filters('quickcab_vehicle_get_suitcase_space', $this->suitcase_space, $this->get_ID());
  }

  /**
   * Return the number of this vehicle type owned
   *
   * @since  1.0
   * @return int  vehicles owned
   */
  public function get_number_owned() {
    return apply_filters('quickcab_vehicle_get_number_owned', $this->number_owned, $this->get_ID());
  }

  /**
   * Return the calendar color of this vehicle
   *
   * @since  1.0
   * @return int  vehicles owned
   */
  public function get_calendar_color() {
    return apply_filters('quickcab_vehicle_get_calendar_color', $this->calendar_color, $this->get_ID());
  }

  /**
   * Return the distance tiers
   *
   * @since  1.0
   * @return array  array of distance tiers
   */
  public function get_distance_tiers() {
    global $wpdb;

    return $wpdb->get_results($wpdb->prepare(
      "SELECT * FROM `" . $wpdb->prefix . "quickcab_tiered_pricing` WHERE `vehicle_id` = %d AND `type` = 'distance' ORDER BY `value_from` ASC;",
      $this->get_ID()
    ));
  }

  /**
   * Return the hourly tiers
   *
   * @since  1.0
   * @return array  array of hourly tiers
   */
  public function get_hourly_tiers() {
    global $wpdb;

    return $wpdb->get_results($wpdb->prepare(
      "SELECT * FROM `" . $wpdb->prefix . "quickcab_tiered_pricing` WHERE `vehicle_id` = %d AND `type` = 'hourly' ORDER BY `value_from` ASC;",
      $this->get_ID()
    ));
  }

  /**
   * Return the hourly pricing tiers
   *
   * @since  1.0
   * @return array  the hourly pricing tiers
   */
  public function get_price_per_hour_tiered() {
    $formatted_hourly_tiers = array();
    $tiers = $this->get_hourly_tiers();

    foreach ( $tiers as $hourly_tier ) {
      array_push(
        $formatted_hourly_tiers,
        array(
          'value_from' => $hourly_tier->value_from,
          'value_to' => $hourly_tier->value_to,
          'price' => $hourly_tier->price,
          'range' => $hourly_tier->value_from . "-" . $hourly_tier->value_to
        )
      );
    }

    return apply_filters('quickcab_vehicle_get_price_per_hour_tiered', $formatted_hourly_tiers, $this->get_ID());
  }

  /**
   * Return the distance pricing tiers
   *
   * @since  1.0
   * @return array  the distance pricing tiers
   */
  public function get_price_per_distance_tiered() {
    $formatted_distance_tiers = array();
    $tiers = $this->get_distance_tiers();

    foreach ( $tiers as $distance_tier ) {
      array_push(
        $formatted_distance_tiers,
        array(
          'id' => $distance_tier->id,
          'value_from' => $distance_tier->value_from,
          'value_to' => $distance_tier->value_to,
          'price' => $distance_tier->price,
          'range' => $distance_tier->value_from . "-" . $distance_tier->value_to,
          'wpdb_object' => $distance_tier
        )
      );
    }

    return apply_filters('quickcab_vehicle_get_price_per_distance_tiered', $formatted_distance_tiers, $this->get_ID());
  }

  public function get_price_per_distance_tiered_for_price_calculation() {
    return apply_filters('quickcab_vehicle_get_price_per_distance_tiered_for_price_calculation', $this->get_price_per_distance_tiered());
  }

  /**
   * Gets all QuickCab_Vehicles
   *
   * @since  1.0
   * @return array  array of all vehicles
   */
  public static function find_all() {
    $vehicles = array();

    $post_search = new WP_Query(array(
      'post_type' => 'quickcab_vehicle',
      'post_staus' => array('publish'),
      'posts_per_page' => -1
    ));

    $unacceptable_status = apply_filters('quickcab_vehicle_unacceptable_status', array(
      'auto-draft', 'future', 'draft', 'private', 'trash'
    ));

    foreach ( $post_search->posts as $vehicle ) {
      if ( in_array($vehicle->post_status, $unacceptable_status) ) {
        continue;
      }

      $vehicles[$vehicle->ID] = new QuickCab_Vehicle($vehicle->ID);
    }

    return $vehicles;
  }

  /**
   * Fetch the names of all QuickCab_Vehicles
   *
   * @since  1.0
   * @return array  array of all vehicle names
   */
  public static function find_all_name() {
    $vehicles = QuickCab_Vehicle::find_all();
    $formatted_vehicles = array();

    foreach ( $vehicles as $vehicle ) {
      $formatted_vehicles[$vehicle->get_ID()] = $vehicle->post_title;
    }

    return $formatted_vehicles;
  }

  /**
   * Get the meta data for the vehicle
   *
   * @since  1.0
   * @param  string  $key     the key to fetch
   * @param  boolean $single  get single data or array
   * @return [array|string]   value of meta data if single = true, array if not
   */
  public function get_meta($key = '', $single = true) {
    $value = get_post_meta($this->get_ID(), $this->meta_prefix . $key, $single);
    return apply_filters('quickcab_vehicle_get_meta_' . $key, $value, $this->get_ID(), $single);
  }

  /**
   * Update the meta data for the vehicle
   *
   * @since  1.0
   * @param  string $key         the key to update
   * @param  string $value       the new value
   * @param  string $prev_value  the old value
   * @return [int|bool]          meta ID if key didn't exist, true on success, false on fail
   */
  public function update_meta($key = '', $value = '', $prev_value = '') {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $meta_key = $this->meta_prefix . $key;

    $value = apply_filters('quickcab_vehicle_update_meta_' . $key, $value, $this->get_ID());

    $updated = update_post_meta($this->get_ID(), $meta_key, $value, $prev_value);

    if ( $updated ) {
      $this->$key = $value;
    }

    return $updated;
  }

  /**
   * Delete the meta data for the vehicle
   *
   * @since  1.0
   * @param  string $key  the key to delete
   * @return bool         success or not
   */
  public function delete_meta($key) {
    if ( empty($key) || $key == '' ) {
      return false;
    }

    $key = $this->meta_prefix . $key;

    return delete_post_meta($this->get_ID(), $key);
  }

  /**
   * Get the pricing info for a given journey
   *
   * @since  1.0
   * @param  QuickCab_Journey $journey  the journey to be quoted for
   * @return array                the pricing info for the journey
   */
  public function calculate_price_info($journey) {
    $units = quickcab_get_option('distance_units');

    $price_info = array(
      'total_unformatted' => 0,
      'total_formatted' => '',
      'breakdown' => array()
    );

    // uses base fare and duration
    $standard_pricing_modes = apply_filters('quickcab_standard_pricing_modes', array('per-distance'));

    if ( in_array($this->get_pricing_structure(), $standard_pricing_modes) ) {
      // starting fare
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'base_fare',
        esc_html__('Base Fare', 'quickcab'),
        $this->get_starting_price(),
        ''
      );

      // duration
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'duration',
        esc_html__('Duration', 'quickcab'),
        $journey->get_duration_in_minutes() * $this->get_price_per_minute(),
        quickcab_format_price($this->get_price_per_minute()) . ' / ' . esc_html__('minute', 'quickcab')
      );
    }

    if ( $journey->get_booking_form()->enable_passengers_field() ) {
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'passengers',
        esc_html__('Passengers', 'quickcab'),
        $journey->get_no_of_occupants() * $this->get_price_per_occupant(),
        quickcab_format_price($this->get_price_per_occupant()) . ' / ' . esc_html__('occupant', 'quickcab')
      );
    }

    if ( $journey->get_booking_form()->enable_suitcases_field() ) {
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'suitcases',
        esc_html__('Suitcases', 'quickcab'),
        $journey->get_no_of_suitcases() * $this->get_price_per_suitcase(),
        quickcab_format_price($this->get_price_per_suitcase()) . ' / ' . esc_html__('luggage', 'quickcab')
      );
    }

    $price_info = quikcab_add_to_price_info(
      $price_info,
      'waiting_time',
      esc_html__('Waiting Time', 'quickcab'),
      // get_extra_waiting_time() in minutes
      $journey->get_extra_waiting_time() * $this->get_price_per_waiting_min(),
      quickcab_format_price($this->get_price_per_waiting_unit()) . ' / ' . $this->get_waiting_unit_i18n()
    );

    if ( $this->get_pricing_structure() == 'per-distance' ) {
      // Cost per unit distance (mile or kilometer)
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'distance',
        esc_html__('Distance', 'quickcab'),
        $journey->get_distance_in_given_unit() * $this->get_price_per_distance(),
        quickcab_format_price($this->get_price_per_minute()) . ' / ' . quickcab_singular_units_word(quickcab_get_option('distance_units'))
      );
    }

    if ( $this->get_pricing_structure() == 'per-hour' ) {
      // cost per hour
      $price_info = quikcab_add_to_price_info(
        $price_info,
        'duration',
        esc_html__('Duration', 'quickcab'),
        ceil($journey->get_duration_in_hours()) * $this->get_price_per_hour(),
        quickcab_format_price($this->get_price_per_hour()) . ' / ' . esc_html__('hour', 'quickcab')
      );
    }

    $price_info = apply_filters('quickcab_price_after_all_vehicle_costs_calculation', $price_info, $this, $journey);

    if ( $journey->is_return() ) {
      $return_price_info = $this->calculate_price_info($journey->get_return_journey());
      $price_info = quickcab_merge_price_info($price_info, $return_price_info);

      $return_amount = $return_price_info['total_unformatted'];

      $return_fare_discount = apply_filters(
        'quickcab_return_fare_discount',
        floatval(quickcab_get_option('return_discount_rate')) / 100,
        $this,
        $journey
      );

      $price_info = quikcab_add_to_price_info(
        $price_info,
        'return_fare_discount',
        esc_html__('Return Fare Discount', 'quickcab'),
        ($return_amount * $return_fare_discount) * -1,
        ''
      );
    }

    // Check for maximum fare
    if (
      quickcab_get_option('enforce_max_fare') == 'on' &&
      $price_info['total_unformatted'] > floatval(quickcab_get_option('maximum_fare_amount'))
    ) {
      $difference = floatval(quickcab_get_option('maximum_fare_amount')) - $price_info['total_unformatted'];

      $price_info = quikcab_add_to_price_info(
        $price_info,
        'max_fare_discount',
        esc_html__('Maximum Fare Discount', 'quickcab'),
        $difference,
        ''
      );
    }

    $price_info = apply_filters(
      'quickcab_price_after_max_fare_calculation',
      $price_info,
      $this,
      $journey
    );

    // Check for minimum fare
    if (
      quickcab_get_option('enforce_min_fare') == 'on' &&
      $price_info['total_unformatted'] < floatval(quickcab_get_option('minimum_fare_amount'))
    ) {
      $difference = floatval(quickcab_get_option('minimum_fare_amount')) - $price_info['total_unformatted'];

      $price_info = quikcab_add_to_price_info(
        $price_info,
        'min_fare_surcharge',
        esc_html__('Minimum Fare Surcharge', 'quickcab'),
        $difference,
        ''
      );

      $total_fare = floatval(quickcab_get_option('minimum_fare_amount'));
    }

    return apply_filters('quickcab_price_calculation_return', $price_info, $this, $journey);
  }

  /**
   * Get the url of the vehicle image
   *
   * @since  1.0
   * @param  string $size  the size of the image
   * @return string        the image url
   */
  public function get_image_url($size = '') {
    return get_the_post_thumbnail_url($this->get_ID(), $size);
  }

  public function get_pretty_price() {
  }

  /**
   * Gets all the features of the current vehicle, formats them into an array
   *
   * @since  1.0
   * @return array  array of features
   */
  public function get_all_features() {
    $formatted_features = array();
    $features = get_the_terms($this->get_ID(), 'vehicle_features');

    if ( false === $features ) {
      return array();
    }

    foreach ( $features as $feature ) {
      array_push($formatted_features, array(
        'name'        => $feature->name,
        'description' => $feature->description
      ));
    }

    return $formatted_features;
  }

  /**
   * Is the vehicle acceptable for the given journey?
   *
   * @since  1.0
   * @param  QuickCab_Journey $journey  the journey to be tested
   * @return boolean              is it acceptable?
   */
  public function is_acceptable_for_journey($journey) {
    // defaults to true but allows filter to override
    $acceptable_filter = apply_filters('quickcab_vehicle_acceptable_for_journey', true, $this->get_ID(), $journey);

    if ( !$acceptable_filter ) {
      return false;
    }

    if ( $journey->get_booking_form()->enable_passengers_field() && $journey->get_no_of_occupants() > $this->get_occupant_space() ) {
      return false;
    }

    if ( $journey->get_booking_form()->enable_suitcases_field() && $journey->get_no_of_suitcases() > $this->get_suitcase_space() ) {
      return false;
    }

    if ( !$this->has_unbooked_vehicles_for_journey($journey) ) {
      return false;
    }

    return true;
  }

  /**
   * Checks if the current vehicle is available to be used for the passed
   * journey (i.e. not all vehicles are on another journey).
   *
   * @since   1.0
   * @param   QuickCab_Journey     $journey  the journey to be checked
   * @return  boolean                  is the vehicle fully booked?
   */
  private function has_unbooked_vehicles_for_journey($journey) {
    $available_vehicles = $this->get_number_owned();
    $orders_for_vehicle = QuickCab_Order::find_by_meta_key('vehicle_id', $this->get_ID());

    foreach ( $orders_for_vehicle as $order ) {
      $order_start = $order->get_departure_timestamp();
      $order_end = $order->get_estimated_arrival_timestamp();
      $journey_start = $journey->get_vehicle_activation_timestamp();
      $journey_end = $journey->get_vehicle_deactivation_timestamp();

      // if we factor in time from base address then change start and end times
      if ( quickcab_get_option('use_base_location_for_availability') == 'on' ) {
        $current_base_location = quickcab_get_option('vehicle_base_location');

        // if base location has changed since order, recalculate
        if ( $current_base_location !== $order->get_base_location() ) {
          $results = quickcab_set_base_transit_times($order->get_ID());
          $order_start -= $results['base_to_collection'];
          $order_end += $results['destination_to_base'];
        } else {
          $order_start -= $order->get_base_to_collection_time();
          $order_end += $order->get_destination_to_base_time();
        }
      }

      // if the order ends in the past, skip
      if ( $order_end < time() && !$order->is_return() ) {
        continue;
      }

      // if the order and journey departure are more than 4 days apart, skip
      if ( abs($journey->get_departure_time() - $order->get_departure_timestamp()) > (60 * 60 * 24 * 3) && !$order->is_return() ) {
        continue;
      }

      // if order overlaps with the journey, 1 less available vehicle
      if ( $order_start <= $journey_end && $order_end >= $journey_start ) {
        $available_vehicles--;
      }

      if ( !$order->is_return() ) {
        continue;
      }

      // now do same for return journey
      $order_start = $order->get_return_departure_timestamp();
      $order_end = $order->get_estimated_return_arrival_timestamp();

      if ( quickcab_get_option('use_base_location_for_availability') == 'on' ) {
        $current_base_location = quickcab_get_option('vehicle_base_location');

        // reuse previously calculate vals but swap order since its return
        $order_start -= $order->get_destination_to_base_time();
        $order_end += $order->get_base_to_collection_time();
      }

      // if the order ends in the past, skip
      if ( $order_end < time() ) {
        continue;
      }

      // if the order and journey departure are more than 4 days apart, skip
      if ( abs($journey->get_departure_time() - $order->get_return_departure_timestamp()) > (60 * 60 * 24 * 3) ) {
        continue;
      }

      // if order overlaps with the journey, 1 less available vehicle
      if ( $order_start <= $journey_end && $order_end >= $journey_start ) {
        $available_vehicles--;
      }
    }

    return $available_vehicles > 0;
  }

  /**
   * Populates the default WordPress Post fields to QuickCab_Vehicle
   *
   * @param  WP_Post $post  the WP_Post
   * @return bool           if the save was successful
   */
  private function populate_post_fields($post) {
    if ( !is_object($post) || !is_a($post, 'WP_Post') || $post->post_type !== 'quickcab_vehicle' ) {
      return false;
    }

    foreach ( $post as $key => $value ) {
      $this->$key = $value;
    }

    return true;
  }

  /**
   * Setup the object variables
   *
   * @since  1.0
   * @param  WP_Post $vehicle  the vehicle WP_Post
   * @return bool              did setup succeed?
   */
  private function setup_vehicle($vehicle = null) {

    if ( is_null($vehicle) || !is_a($vehicle, 'WP_Post') || $vehicle->post_type !== 'quickcab_vehicle' ) {
      return false;
    }

    do_action('quickcab_before_setup_vehicle', $this, $vehicle);

    $this->ID = $vehicle->ID;
    $this->wp_post = $vehicle;

    $this->name                         = $this->setup_name();
    $this->pricing_structure            = $this->setup_pricing_structure();
    $this->starting_price               = $this->setup_starting_price();
    $this->price_per_distance           = $this->setup_price_per_distance();
    $this->price_per_distance_from_base = $this->setup_price_per_distance_from_base();
    $this->price_per_hour               = $this->setup_price_per_hour();
    $this->price_per_minute             = $this->setup_price_per_minute();
    $this->price_per_occupant           = $this->setup_price_per_occupant();
    $this->price_per_suitcase           = $this->setup_price_per_suitcase();
    $this->price_per_waiting_unit       = $this->setup_price_per_waiting_unit();
    $this->waiting_unit                 = $this->setup_waiting_unit();
    $this->occupant_space               = $this->setup_occupant_space();
    $this->suitcase_space               = $this->setup_suitcase_space();
    $this->number_owned                 = $this->setup_number_owned();
    $this->calendar_color               = $this->setup_calendar_color();

    $this->populate_post_fields($this->wp_post);

    do_action('quickcab_after_setup_vehicle', $this, $vehicle);

    return true;
  }

  /**
   * Setup the name of the vehicle
   *
   * @since  1.0
   * @return string  the vehicle name
   */
  private function setup_name() {
    return get_the_title($this->get_ID());
  }

  /**
   * Setup the pricing structure of the vehicle
   *
   * @since  1.0
   * @return string  the vehicle pricing structure
   */
  private function setup_pricing_structure() {
    return $this->get_meta('pricing_structure');
  }

  /**
   * Setup the starting price of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle starting price
   */
  private function setup_starting_price() {
    return (float) $this->get_meta('starting_price');
  }

  /**
   * Setup the price per distance of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per distance
   */
  private function setup_price_per_distance() {
    return (float) $this->get_meta('price_per_distance');
  }

  /**
   * Setup the price per distance (base to collection) of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per distance (base to collection)
   */
  private function setup_price_per_distance_from_base() {
    return (float) $this->get_meta('price_per_distance_from_base');
  }

  /**
   * Setup the price per hour of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per hour
   */
  private function setup_price_per_hour() {
   return (float) $this->get_meta('price_per_hour');
  }

  /**
   * Setup the price per minute of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per minute
   */
  private function setup_price_per_minute() {
   return (float) $this->get_meta('price_per_minute');
  }

  /**
   * Setup the price per occupant of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per occupant
   */
  private function setup_price_per_occupant() {
   return (float) $this->get_meta('price_per_occupant');
  }

  /**
   * Setup the price per suitcase of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per suitcase
   */
  private function setup_price_per_suitcase() {
   return (float) $this->get_meta('price_per_suitcase');
  }

  /**
   * Setup the price per waiting hour of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per waiting hour
   */
  private function setup_price_per_waiting_unit() {
    return (float) $this->get_meta('price_per_waiting_unit');
  }

  /**
   * Setup the price per waiting hour of the vehicle
   *
   * @since  1.0
   * @return float  the vehicle price per waiting hour
   */
  private function setup_waiting_unit() {
    return $this->get_meta('waiting_unit');
  }

  /**
   * Setup the occupant space of the vehicle
   *
   * @since  1.0
   * @return int  the vehicle occupant space
   */
  private function setup_occupant_space() {
   return (int) $this->get_meta('occupant_space');
  }

  /**
   * Setup the suitcase space of the vehicle
   *
   * @since  1.0
   * @return int  the vehicle suitcase space
   */
  private function setup_suitcase_space() {
   return (int) $this->get_meta('suitcase_space');
  }

  /**
   * Setup the number of vehicles owned for this vehicle
   *
   * @since  1.0
   * @return int  the number of vehicles owned
   */
  private function setup_number_owned() {
    return (int) $this->get_meta('number_owned');
  }

  /**
   * Setup the calendar color for this vehicle
   *
   * @since  1.0
   * @return string  the calendar color
   */
  private function setup_calendar_color() {
    return $this->get_meta('calendar_color');
  }
}
