<?php

if ( !defined('ABSPATH') ) exit;

/**
 * On every page load check if form has been submitted.
 *
 * @since   1.0
 * @return  void
 */
function quickcab_check_for_new_order() {
  if ( !empty($_POST) && isset($_POST['quickcab_form']['id']) ) {
    quickcab_process_purchase_form();
  }
}
add_action('wp_loaded', 'quickcab_check_for_new_order');

/**
 * Gets all information from the form and initiates the payment
 *
 * @since  1.0
 * @return void
 */
function quickcab_process_purchase_form() {
  global $woocommerce;

  do_action('quickcab_before_process_purchase', $_POST);

  $form_id = wp_unslash($_POST['quickcab_form']['id']);
  $booking_form = new QuickCab_Booking_Form($form_id);

  if ( !quickcab_validate_form_info($booking_form) ) {
    error_log('Error validating form information');
    return false;
  }

  // recalculate quote to verify price and get details
  $journey_options = array(
    'departure_date' => wp_unslash($_POST['quickcab_form']['departure_date']),
    'departure_time' => wp_unslash($_POST['quickcab_form']['departure_time']),
    'origin' => array(
      'lat' => wp_unslash($_POST['quickcab_form']['origin']['lat']),
      'lng' => wp_unslash($_POST['quickcab_form']['origin']['lng']),
      'address' => wp_unslash($_POST['quickcab_form']['origin']['address'])
    ),
    'destination' => array(
      'lat' => wp_unslash($_POST['quickcab_form']['destination']['lat']),
      'lng' => wp_unslash($_POST['quickcab_form']['destination']['lng']),
      'address' => wp_unslash($_POST['quickcab_form']['destination']['address'])
    ),
    'booking_form' => $booking_form
  );

  if ( $booking_form->enable_direction_field() ) {
    $journey_options['direction'] = wp_unslash($_POST['quickcab_form']['direction']);
  } else {
    $journey_options['direction'] = 'one_way';
  }

  if ( $booking_form->enable_passengers_field() ) {
    $journey_options['occupants'] = intval($_POST['quickcab_form']['occupants']);
  } else {
    $journey_options['occupants'] = 0;
  }

  if ( $booking_form->enable_suitcases_field() ) {
    $journey_options['suitcases'] = intval($_POST['quickcab_form']['suitcases']);
  } else {
    $journey_options['suitcases'] = 0;
  }

  if ( $booking_form->enable_extra_waiting_time() ) {
    $journey_options['extra_waiting_time'] = wp_unslash($_POST['quickcab_form']['extra_waiting_time']);
  }

  if ( isset($_POST['quickcab_form']['waypoints']) ) {
    $journey_options['waypoints'] = array_map(function($waypoint) {
      return array(
        'lat' => $waypoint['lat'],
        'lng' => $waypoint['lng'],
        'address' => $waypoint['address']
      );
    }, $_POST['quickcab_form']['waypoints']);
  }

  if ( $_POST['quickcab_form']['direction'] == 'return' ) {
    $journey_options['return_departure_date'] = wp_unslash($_POST['quickcab_form']['return_departure_date']);
    $journey_options['return_departure_time'] = wp_unslash($_POST['quickcab_form']['return_departure_time']);
  }

  $journey = new QuickCab_Journey($journey_options);
  $vehicle = new QuickCab_Vehicle(intval($_POST['quickcab_form']['vehicle_id']));

  if ( $journey->has_error() ) {
    error_log('Error in journey calculation');
    return false;
  }

  if ( !$vehicle->is_acceptable_for_journey($journey) ) {
    error_log('Vehicle is not acceptable for journey');
    return false;
  }

  $price_object = quickcab_make_price_object($vehicle, $journey);

  $recalculated_price = floatval($price_object['quote']['total_breakdown']['total']['amount_unformatted']);
  $given_price = floatval($_POST['quickcab_form']['total']);

  if ( abs($recalculated_price - $given_price) > 0.00001 ) {
    error_log('Incorrect price passed from form');
    return false;
  }

  $woocommerce->session->set('quickcab_total', $recalculated_price);

  $departure_time = quickcab_get_datetime_object(
    $_POST['quickcab_form']['departure_date'],
    $_POST['quickcab_form']['departure_time']
  );

  if ( !$departure_time ) {
    error_log('Error parsing departure time');
    return false;
  }

  $purchase_data = array(
    'starting_coords_lat'         => wp_unslash($_POST['quickcab_form']['origin']['lat']),
    'starting_coords_long'        => wp_unslash($_POST['quickcab_form']['origin']['lng']),
    'starting_address'            => wp_unslash($_POST['quickcab_form']['origin']['address']),
    'destination_coords_lat'      => wp_unslash($_POST['quickcab_form']['destination']['lat']),
    'destination_coords_long'     => wp_unslash($_POST['quickcab_form']['destination']['lng']),
    'destination_address'         => wp_unslash($_POST['quickcab_form']['destination']['address']),
    'vehicle_id'                  => intval($_POST['quickcab_form']['vehicle_id']),
    'departure_time'              => $departure_time->format('Y-m-d H:i:s'),
    'is_return'                   => $_POST['quickcab_form']['direction'] == 'return' ? 1 : 0,
    'price_breakdown'             => $price_object['quote']['breakdown'],
    'duration_in_seconds'         => $journey->get_duration_in_seconds(),
    'duration_in_words'           => $journey->get_duration_in_words(),
    'duration_in_seconds_traffic' => $journey->get_duration_in_seconds(),
    'duration_in_words_traffic'   => $journey->get_duration_in_words(),
    'distance_in_metres'          => $journey->get_distance_in_meters(),
    'distance_in_words'           => $journey->get_distance_in_words(),
    'form_id'                     => wp_unslash($_POST['quickcab_form']['id'])
  );

  if ( $booking_form->enable_extra_waiting_time() ) {
    $purchase_data['extra_waiting_time'] = intval($_POST['quickcab_form']['extra_waiting_time']);
  }

  if ( $booking_form->enable_passengers_field() ) {
    $purchase_data['occupants'] = intval($_POST['quickcab_form']['occupants']);
  }

  if ( $booking_form->enable_suitcases_field() ) {
    $purchase_data['suitcases'] = intval($_POST['quickcab_form']['suitcases']);
  }

  if ( isset($_POST['quickcab_form']['waypoints']) ) {
    $purchase_data['waypoints'] = array_map(function($waypoint) {
      return array(
        'lat' => wp_unslash($waypoint['lat']),
        'lng' => wp_unslash($waypoint['lng']),
        'address' => wp_unslash($waypoint['address'])
      );
    }, $_POST['quickcab_form']['waypoints']);
  }

  // if its a return journey, also add return departure times
  if ( $_POST['quickcab_form']['direction'] == 'return' ) {
    $ret_departure_time = quickcab_get_datetime_object(
      $_POST['quickcab_form']['return_departure_date'],
      $_POST['quickcab_form']['return_departure_time']
    );

    if ( !$ret_departure_time ) {
      error_log('Error parsing return departure time');
      return false;
    }

    $purchase_data['return_departure_time'] = $ret_departure_time->format('Y-m-d H:i:s');

    $return_journey = $journey->get_return_journey();

    if ( $return_journey->has_error() ) {
      error_log('Error in return journey calculation');
      return false;
    }

    $purchase_data['return_duration_in_seconds'] = $return_journey->get_duration_in_seconds();
    $purchase_data['return_duration_in_words'] = $return_journey->get_duration_in_words();
    $purchase_data['return_duration_in_seconds_traffic'] = $return_journey->get_duration_in_seconds();
    $purchase_data['return_duration_in_words_traffic'] = $return_journey->get_duration_in_words();
  }

  quickcab_set_checkout_journey_info($purchase_data);

  quickcab_send_to_checkout();
  wp_die();
}

function quickcab_send_to_checkout() {
  global $woocommerce;

  $product_id = quickcab_get_or_create_wc_product();
  $woocommerce->cart->add_to_cart($product_id);

  if ( $woocommerce->cart->get_cart_contents_count() == 0 ) {
    error_log('Error: there are no items in the WooCommerce cart');
  }

  wp_redirect(wc_get_checkout_url());
  exit;
}

function quickcab_calculate_checkout_price($cart) {
  global $woocommerce;

  // Done this way to make it secure so user cant change value in post request
  foreach ( $cart->cart_contents as $key => $value ) {
    if ( $value['product_id'] == quickcab_get_or_create_wc_product() ) {
      $value['data']->set_price($woocommerce->session->get('quickcab_total'));
    }
  }
}
add_action('woocommerce_before_calculate_totals', 'quickcab_calculate_checkout_price');

function quickcab_get_datetime_object($date, $time) {
  $timezone = new DateTimeZone(quickcab_get_wordpress_timezone());
  $ret_departure_time = DateTime::createFromFormat("d/m/Y H:i", $date . ' ' . $time, $timezone);

  return $ret_departure_time ? $ret_departure_time : false;
}

function quickcab_set_checkout_journey_info($purchase_data) {
  global $woocommerce;
  
  if ( isset($purchase_data['occupants']) ) {
    $woocommerce->session->set('quickcab_occupants', $purchase_data['occupants']);
  }

  if ( isset($purchase_data['suitcases']) ) {
    $woocommerce->session->set('quickcab_suitcases', $purchase_data['suitcases']);
  }

  if ( isset($purchase_data['extra_waiting_time']) ) {
    $woocommerce->session->set('quickcab_waiting_time', $purchase_data['extra_waiting_time']);
  }

  if ( isset($purchase_data['waypoints']) ) {
    $waypoints = $purchase_data['waypoints'];

    $woocommerce->session->set('quickcab_waypoints_count', count($waypoints));

    for ( $i = 0; $i < count($waypoints); $i++ ) {
      $woocommerce->session->set('quickcab_waypoint_' . $i . '_lat', $waypoints[$i]['lat']);
      $woocommerce->session->set('quickcab_waypoint_' . $i . '_lng', $waypoints[$i]['lng']);
      $woocommerce->session->set('quickcab_waypoint_' . $i . '_address', $waypoints[$i]['address']);
    }
  }

  $uses_waypoints = isset($purchase_data['waypoints']) && count($purchase_data['waypoints']) > 0;

  $woocommerce->session->set('quickcab_uses_waypoints', $uses_waypoints);
  $woocommerce->session->set('quickcab_vehicle_id', $purchase_data['vehicle_id']);
  $woocommerce->session->set('quickcab_departure_time', $purchase_data['departure_time']);
  $woocommerce->session->set('quickcab_is_return', $purchase_data['is_return']);
  $woocommerce->session->set('quickcab_duration_in_seconds', $purchase_data['duration_in_seconds']);
  $woocommerce->session->set('quickcab_duration_in_words', $purchase_data['duration_in_words']);
  $woocommerce->session->set('quickcab_duration_in_seconds_traffic', $purchase_data['duration_in_seconds_traffic']);
  $woocommerce->session->set('quickcab_duration_in_words_traffic', $purchase_data['duration_in_words_traffic']);
  $woocommerce->session->set('quickcab_distance_in_metres', $purchase_data['distance_in_metres']);
  $woocommerce->session->set('quickcab_distance_in_words', $purchase_data['distance_in_words']);
  $woocommerce->session->set('quickcab_starting_coords_lat', $purchase_data['starting_coords_lat']);
  $woocommerce->session->set('quickcab_starting_coords_long', $purchase_data['starting_coords_long']);
  $woocommerce->session->set('quickcab_starting_address', $purchase_data['starting_address']);
  $woocommerce->session->set('quickcab_destination_coords_lat', $purchase_data['destination_coords_lat']);
  $woocommerce->session->set('quickcab_destination_coords_long', $purchase_data['destination_coords_long']);
  $woocommerce->session->set('quickcab_destination_address', $purchase_data['destination_address']);
  $woocommerce->session->set('quickcab_form_id', $purchase_data['form_id']);

  if ( $purchase_data['is_return'] ) {
    $woocommerce->session->set('quickcab_return_departure_time', $purchase_data['return_departure_time']);
    $woocommerce->session->set('quickcab_return_duration_in_seconds', $purchase_data['return_duration_in_seconds']);
    $woocommerce->session->set('quickcab_return_duration_in_words', $purchase_data['return_duration_in_words']);
    $woocommerce->session->set('quickcab_return_duration_in_seconds_traffic', $purchase_data['return_duration_in_seconds_traffic']);
    $woocommerce->session->set('quickcab_return_duration_in_words_traffic', $purchase_data['return_duration_in_words_traffic']);
  }
}

/**
 * Validate the $_POST data from the form
 *
 * @since  1.0
 * @return bool          valid data?
 */
function quickcab_validate_form_info($booking_form) {

  $required_fields = array(
    array('quickcab_form', 'origin', 'lat'),
    array('quickcab_form', 'origin', 'lng'),
    array('quickcab_form', 'origin', 'address'),
    array('quickcab_form', 'destination', 'lat'),
    array('quickcab_form', 'destination', 'lng'),
    array('quickcab_form', 'destination', 'address'),
    array('quickcab_form', 'vehicle_id'),
    array('quickcab_form', 'direction'),
    array('quickcab_form', 'departure_date'),
    array('quickcab_form', 'departure_time'),
    array('quickcab_form', 'id')
  );

  if ( $booking_form->enable_passengers_field() ) {
    array_push($required_fields, array('quickcab_form', 'occupants'));
  }

  if ( $booking_form->enable_suitcases_field() ) {
    array_push($required_fields, array('quickcab_form', 'suitcases'));
  }

  if ( $_POST['quickcab_form']['direction'] == 'return' ) {
    $required_fields = array_merge($required_fields, array(
      array('quickcab_form', 'return_departure_date'),
      array('quickcab_form', 'departure_time')
    ));
  }

  foreach ( $required_fields as $field ) {
    if ( !quickcab_validate_form_field($field) ) {
      return false;
    }
  }

  return true;
}

function quickcab_validate_form_field($field) {
  $path = $field;

  if ( is_string($field) ) {
    $path = [$field];
  }

  $current = $_POST;

  for ( $i = 0; $i < count($path); $i++ ) {
    if ( !isset($current[$path[$i]]) ) {
      return false;
    }

    $current = $current[$path[$i]];
  }

  return strlen($current) > 0;
}
