<?php

if ( !defined('ABSPATH') ) exit;

/**
 * Called when the AJAX request for prices is submitted
 *
 * @since  1.0
 * @return void
 */
function quickcab_get_vehicle_prices_ajax() {
  echo json_encode(quickcab_get_prices($_POST));
  wp_die();
}
add_action('wp_ajax_quickcab_get_quotes', 'quickcab_get_vehicle_prices_ajax');
add_action('wp_ajax_nopriv_quickcab_get_quotes', 'quickcab_get_vehicle_prices_ajax');

/**
 * Get the prices of a journey given the journey options
 *
 * @since  1.0
 * @param  array  $options  the options of the journey
 * @return array            prices
 */
function quickcab_get_prices($options) {
  global $woocommerce;

  $booking_form = new QuickCab_Booking_Form($options['quickcab_form']['id']);

  $journey_options = array(
    'departure_date' => $options['quickcab_form']['departure_date'],
    'departure_time' => $options['quickcab_form']['departure_time'],
    'extra_waiting_time' => $options['quickcab_form']['extra_waiting_time'],
    'origin' => array(
      'lat' => $options['quickcab_form']['origin']['lat'],
      'lng' => $options['quickcab_form']['origin']['lng'],
      'address' => $options['quickcab_form']['origin']['address']
    ),
    'destination' => array(
      'lat' => $options['quickcab_form']['destination']['lat'],
      'lng' => $options['quickcab_form']['destination']['lng'],
      'address' => $options['quickcab_form']['destination']['address']
    ),
    'booking_form' => $booking_form
  );

  if ( $booking_form->enable_direction_field() ) {
    $journey_options['direction'] = $options['quickcab_form']['direction'];

    if ( $options['quickcab_form']['direction'] == 'return' ) {
      $journey_options['return_departure_date'] = $options['quickcab_form']['return_departure_date'];
      $journey_options['return_departure_time'] = $options['quickcab_form']['return_departure_time'];
    }
  }

  if ( $booking_form->enable_passengers_field() ) {
    $journey_options['occupants'] = intval($options['quickcab_form']['occupants']);
  }

  if ( $booking_form->enable_suitcases_field() ) {
    $journey_options['suitcases'] = intval($options['quickcab_form']['suitcases']);
  }

  if ( $booking_form->enable_waypoints() && isset($options['quickcab_form']['waypoints']) ) {
    $journey_options['waypoints'] = array_map(function($waypoint) {
      return array(
        'lat' => $waypoint['lat'],
        'lng' => $waypoint['lng'],
        'address' => $waypoint['address']
      );
    }, $options['quickcab_form']['waypoints']);
  }

  $journey = new QuickCab_Journey($journey_options);

  if ( $journey->has_error() ) {
    return array(
      'message' => $journey->get_error_message(),
      'can_book' => false,
      'prices' => array()
    );
  }

  if (
    quickcab_get_option('enforce_min_distance') == 'on' &&
    quickcab_get_option('minimum_fare_distance') > 0 &&
    $journey->get_distance_in_given_unit() < floatval(quickcab_get_option('minimum_fare_distance'))
  ) {
    return array(
      'message' => quickcab_get_option('minimum_fare_distance_error_message'),
      'can_book' => false,
      'prices' => array()
    );
  }

  if (
    quickcab_get_option('enforce_max_distance') == 'on' &&
    quickcab_get_option('maximum_fare_distance') > 0 &&
    $journey->get_distance_in_given_unit() > floatval(quickcab_get_option('maximum_fare_distance'))
  ) {
    return array(
      'message' => quickcab_get_option('maximum_fare_distance_error_message'),
      'can_book' => false,
      'prices' => array()
    );
  }

  if ( $journey->is_return() ) {
    if ( $journey->get_departure_time() > $journey->get_return_journey()->get_departure_time() ) {
      return array(
        'message' => esc_html__('The departure date must be before the return departure date.', 'quickcab'),
        'can_book' => false,
        'prices' => array()
      );
    }

    if ( $journey->get_return_journey()->has_error() ) {
      return array(
        'message' => $journey->get_return_journey()->get_error_message(),
        'can_book' => false,
        'prices' => array()
      );
    }
  }

  if ( $journey->get_distance_in_meters() == 0 ) {
    return array(
      'message' => esc_html__('There was an error calculating the journey.', 'quickcab'),
      'can_book' => false,
      'prices' => array()
    );
  }

  $vehicles = QuickCab_Vehicle::find_all();

  $return_value = array(
    'message' => '',
    'can_book' => true,
    'prices' => array()
  );

  foreach ( $vehicles as $vehicle ) {
    if ( $vehicle->is_acceptable_for_journey($journey) ) {
      $price_object = quickcab_make_price_object($vehicle, $journey);
      array_push($return_value['prices'], $price_object);
    }
  }

  // sort quotes by price, low to high
  usort($return_value['prices'], "quickcab_quotes_sort");

  if ( empty($return_value['prices']) ) {
    return array(
      'message'  => quickcab_get_option('no_quotes_error_message'),
      'can_book' => false,
      'prices'   => array()
    );
  }

  return $return_value;
}

function quickcab_ajax_make_quote_selection() {
  global $woocommerce;

  $vehicle_id = intval(sanitize_text_field($_POST['vehicle_id']));
  $woocommerce->session->set('vehicle_id', $vehicle_id);

  wp_die();
}
add_action('wp_ajax_quickcab_make_quote_selection', 'quickcab_ajax_make_quote_selection');
add_action('wp_ajax_nopriv_quickcab_make_quote_selection', 'quickcab_ajax_make_quote_selection');

/**
 * Sort the array of quotes, cheapest first
 *
 * @since   1.0
 * @param   array     $a  price_info item
 * @param   array     $b  price_info item
 * @return  int           the comparison between the prices
 */
function quickcab_quotes_sort($a, $b) {
  return $a['quote']['total_unformatted'] - $b['quote']['total_unformatted'];
}

/**
 * Assemble the price object that is requested via AJAX
 *
 * @since  1.0
 * @param  QuickCab_Vehicle $vehicle  the vehicle to be used for the quote
 * @param  QuickCab_Journey $journey  the journey to travel
 * @return $price               the custom pricing object
 */
function quickcab_make_price_object($vehicle, $journey) {
  $price = array(
    'quote'   => $vehicle->calculate_price_info($journey),
    'vehicle' => array(
      'name'          => $vehicle->get_name(),
      'image_url'     => $vehicle->get_image_url(),
      'max_occupants' => $vehicle->get_occupant_space(),
      'max_suitcases' => $vehicle->get_suitcase_space(),
      'features'      => $vehicle->get_all_features(),
      'id'            => $vehicle->get_ID()
    )
  );

  $price['quote']['breakdown'] = array(
    'journey' => array(
      'title'              => esc_html__('Journey', 'quickcab'),
      'amount_unformatted' => $price['quote']['total_unformatted'],
      'amount_formatted'   => $price['quote']['total_formatted']
    )
  );

  $subtotal = 0.0;

  foreach ( $price['quote']['breakdown'] as $key => $breakdown ) {
    $subtotal += $breakdown['amount_unformatted'];
  }

  // Create new section, push total
  $price['quote']['total_breakdown'] = array(
    'subtotal' => array(
      'title'              => esc_html__('Subtotal', 'quickcab'),
      'amount_unformatted' => $subtotal,
      'amount_formatted'   => quickcab_format_price($subtotal)
    )
  );

  $total = 0.0;

  foreach ( $price['quote']['total_breakdown'] as $key => $total_breakdown ) {
    $total += $total_breakdown['amount_unformatted'];
  }

  // Push total
  $price['quote']['total_breakdown']['total'] = array(
    'title'              => esc_html__('Total', 'quickcab'),
    'amount_unformatted' => $total,
    'amount_formatted'   => quickcab_format_price($total)
  );

  $price['serialized_price_breakdown'] = serialize($price['quote']);

  uksort($price['quote']['total_breakdown'], "quickcab_order_total_breakdown");

  return $price;
}

/**
 * Order the total_breakdown section
 *
 * @since  1.0
 * @param  string $a  a key to compare
 * @param  string $b  a key to compare
 * @return int        result of comparison of $a and $b
 */
function quickcab_order_total_breakdown($a, $b) {
  $order = apply_filters('quickcab_order_total_breakdown', array('subtotal', 'total'));

  $a = array_search($a, $order);
  $b = array_search($b, $order);

  if ( $a === false && $b === false ) {
    return 0;
  } else if ( $a === false ) {
    return 1;
  } else if ( $b === false ) {
    return -1;
  }

  return $a - $b;
}
